//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper16.cpp - Implementation of mapper #2 (Bandai)
//////////////////////////////////////////////////////////////////////

#include "NesMapper016.h"


NesMapper16::NesMapper16(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	// Replace the standard PPU memory object with a VRC4 specific one	
	NesPPU *newPPU = new BandaiPPU();
	newPPU->Init();
	newPPU->VRAM->mbytMirroringType = linkPPU->VRAM->mbytMirroringType;
	newPPU->VRAM->link(this);	
	delete mPpu;
	mPpu = newPPU;
	linkPPU = mPpu;
	mCpu->linkCPU(mPpu, mCpu->mApu, mCpu->mController1, mCpu->mController2, this);


	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();
	intIrqCount = 0;
	blnIrqEnable = false;
}


void NesMapper16::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank to $C000
	}
}


void NesMapper16::write(Uint16 addr, unsigned char data) {	
	
	switch (addr) {
	case 0x6000:
	case 0x7FF0:
	case 0x8000:
		swapCHRBank1(0x0000, data % (bytCHRCount << 3));
		break;

	case 0x6001:
	case 0x7FF1:
	case 0x8001:
		swapCHRBank1(0x0400, data % (bytCHRCount << 3));
		break;
	
	case 0x6002:
	case 0x7FF2:
	case 0x8002:
		swapCHRBank1(0x0800, data % (bytCHRCount << 3));
		break;

	case 0x6003:
	case 0x7FF3:
	case 0x8003:
		swapCHRBank1(0x0C00, data % (bytCHRCount << 3));
		break;
	
	case 0x6004:
	case 0x7FF4:
	case 0x8004:
		swapCHRBank1(0x1000, data % (bytCHRCount << 3));
		break;
	
	case 0x6005:
	case 0x7FF5:
	case 0x8005:
		swapCHRBank1(0x1400, data % (bytCHRCount << 3));		
		break;
		
	case 0x6006:
	case 0x7FF6:
	case 0x8006:
		swapCHRBank1(0x1800, data % (bytCHRCount << 3));
		break;
	
	case 0x6007:
	case 0x7FF7:
	case 0x8007:
		swapCHRBank1(0x1C00, data % (bytCHRCount << 3));
		break;

	case 0x6008:
	case 0x7FF8:
	case 0x8008:
		swapPRGBank16(0x8000, data % bytPRGCount);		
		break;
	
	case 0x6009:
	case 0x7FF9:
	case 0x8009:
		switch (data & 0x03) {
		case 0x00:
			mPpu->VRAM->mbytMirroringType = VERTICAL;
			break;
		case 0x01:			
			mPpu->VRAM->mbytMirroringType = HORIZONTAL;
			break;
		case 0x02:
			mPpu->VRAM->mbytMirroringType = SINGLESCREEN;
			mPpu->VRAM->mintSSMirrorLocation = 0x2000;
			break;
		case 0x03:
			mPpu->VRAM->mbytMirroringType = SINGLESCREEN;
			mPpu->VRAM->mintSSMirrorLocation = 0x2400;
			break;
		}
		break;
	
	case 0x600A:	// IRQ enable
	case 0x7FFA:
	case 0x800A:
		blnIrqEnable = (data & 0x01) ? true : false;
		intIrqCount = intIrqLatch;
		break;

	case 0x600B:	// Low byte of IRQ counter
	case 0x7FFB:
	case 0x800B:
		intIrqLatch = (intIrqLatch & 0xFF00) | data;
		break;
	
	case 0x600C:	// High byte of IRQ counter
	case 0x7FFC:
	case 0x800C:
		intIrqLatch = (intIrqLatch & 0x00FF) | (data << 8);
		break;
	}
/*
	if (addr >= 0x6000) {
	switch (addr & 0x000F) {
	case 0x0:
		swapCHRBank1(0x0000, data % (bytCHRCount << 3));
		break;

	case 0x1:
		swapCHRBank1(0x0400, data % (bytCHRCount << 3));
		break;
	
	case 0x2:
		swapCHRBank1(0x0800, data % (bytCHRCount << 3));
		break;
	
	case 0x3:
		swapCHRBank1(0x0C00, data % (bytCHRCount << 3));
		break;
	
	case 0x4:
		swapCHRBank1(0x1000, data % (bytCHRCount << 3));
		break;
	
	case 0x5:
		swapCHRBank1(0x1400, data % (bytCHRCount << 3));		
		break;
		
	case 0x6:
		swapCHRBank1(0x1800, data % (bytCHRCount << 3));
		break;
	
	case 0x7:
		swapCHRBank1(0x1C00, data % (bytCHRCount << 3));
		break;

	case 0x8:
		swapPRGBank16(0x8000, data % bytPRGCount);		
		break;
	
	case 0x9:
		switch (data & 0x03) {
		case 0x00:
			mPpu->VRAM->mbytMirroringType = HORIZONTAL;
			break;
		case 0x01:
			mPpu->VRAM->mbytMirroringType = VERTICAL;
			break;
		case 0x02:
			mPpu->VRAM->mbytMirroringType = SINGLESCREEN;
			mPpu->VRAM->mintSSMirrorLocation = 0x2000;
			break;
		case 0x03:
			mPpu->VRAM->mbytMirroringType = SINGLESCREEN;
			mPpu->VRAM->mintSSMirrorLocation = 0x2400;
			break;
		}
		break;
	
	case 0xA:	// IRQ enable		
		blnIrqEnable = (data & 0x01) ? true : false;
		intIrqCount = intIrqLatch;
		break;

	case 0xB:	// Low byte of IRQ counter
		intIrqLatch = (intIrqLatch & 0xFF00) | data;
		break;
	
	case 0xC:	// High byte of IRQ counter
		intIrqLatch = (intIrqLatch & 0x00FF) | (data << 8);
		break;
	}
	}
*/
}