//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////////////////////
// NesMapper32.cpp - Implementation of mapper #32 (Irem G-101)
//////////////////////////////////////////////////////////////////////

#include "NesMapper032.h"



NesMapper32::NesMapper32(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU) {
	mCartridge = linkCartridge;
	mCpu = linkCPU;
	mPpu = linkPPU;
	
	bytPRGCount = mCartridge->header.PRGROM_count;
	bytCHRCount = mCartridge->header.CHRROM_count;
	
	initMaps();

	blnSwapLow = true;
}


void NesMapper32::initMemory() {	
	if (bytPRGCount > 0) {
		swapPRGBank16(0x8000, 0);			  // First 16k bank to $8000
		swapPRGBank16(0xC000, bytPRGCount-1); // Last 16k bank to $C000
	}
}

// This is all pretty straight forward
void NesMapper32::write(Uint16 addr, unsigned char data) {		
	switch (addr) {
	case 0x8000:
	case 0x8FFF:
		if (blnSwapLow) {
			swapPRGBank8(0x8000, data % (bytPRGCount << 1) );
		} else {
			swapPRGBank8(0xC000, data % (bytPRGCount << 1) );
		}
		break;

	case 0x9000:
	case 0x9FFF:
		blnSwapLow = !((data & 0x02) >> 1);
		mPpu->VRAM->mbytMirroringType = (data & 0x01) ? HORIZONTAL : VERTICAL;
		break;

	case 0xA000:
	case 0xAFFF:
		swapPRGBank8(0xA000, data % (bytPRGCount << 1) );
		break;
	
	case 0xBFF0:
	case 0xB000:
		swapCHRBank1(0x0000, data % (bytCHRCount << 3));
		break;
	
	case 0xBFF1:
	case 0xB001:
		swapCHRBank1(0x0400, data % (bytCHRCount << 3));
		break;
	
	case 0xBFF2:
	case 0xB002:
		swapCHRBank1(0x0800, data % (bytCHRCount << 3));
		break;
	
	case 0xBFF3:
	case 0xB003:
		swapCHRBank1(0x0C00, data % (bytCHRCount << 3));
		break;
	
	case 0xBFF4:
	case 0xB004:
		swapCHRBank1(0x1000, data % (bytCHRCount << 3));
		break;
	
	case 0xBFF5:
	case 0xB005:
		swapCHRBank1(0x1400, data % (bytCHRCount << 3));
		break;
	
	case 0xBFF6:
	case 0xB006:
		swapCHRBank1(0x1800, data % (bytCHRCount << 3));
		break;
	
	case 0xBFF7:
	case 0xB007:
		swapCHRBank1(0x1C00, data % (bytCHRCount << 3));
		break;
	}
}