//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


////////////////////////////////////////////////////////////////////
// Nes Mapper #64:  Tengen RAMBO-1
// This mapper is used on several unlicensed titles by Tengen, including Shinobi,
// Klax, and Skull & Crossbones
////////////////////////////////////////////////////////////////////

#ifndef NES_MAPPER_64
#define NES_MAPPER_64

#include "../NesBaseMapper.h"


class NesMapper64 : public NesBaseMapper
{
public:
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesMapper64(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *&linkPPU);
	~NesMapper64() { };
	void initMemory();
	void write(Uint16 addr, unsigned char data);

	void saveState(FILE *stream);
	void loadState(FILE *stream);

	unsigned char reg0;
	int intIrqLatch, intIrqCount, intIrqMode, intIrqNext, intIrqReset;
	bool blnIrqEnable;
};


// We need a special PPU class for scanline counting/IRQ generation
class TengenPPU : public NesPPU {
	virtual void calcScanLine() {
		// Draw scanline as usual
		NesPPU::calcScanLine();

		// Get a handle to our mapper object, this is pretty convoluted
		NesMapper64 *mmc = (NesMapper64 *) VRAM->mMapper;
	
		if (mmc->intIrqMode) return;
		mmc->intIrqReset = 0;

		// Do IRQ logic (this is approximate and I don't have any games to test with)
		if (mmc->blnIrqEnable) {
			if (mmc->intIrqCount > 0) mmc->intIrqCount -= 113;
			if (mmc->intIrqCount < 0) {
			
				mmc->mPpu->timers.blnOtherIRQ = true;
				mmc->mPpu->timers.intOtherPix = 0*3;
				mmc->intIrqReset = 1;
				mmc->blnIrqEnable = false;
			}
		}	
	
	}
};


#endif 
