////////////////////////////////////////////////////////////////////
// Nes Mapper #1:  MMC1
// This mapper is used on numerous U.S and Japanese games, inclusing Legend of Zelda, Metroid,
// Rad Racer, Mega Man 2, and many others.
//
// Most of this mapper was written from the excellent MMC1 document written by
// Mathew Richey.
////////////////////////////////////////////////////////////////////

#if !defined(NES_MAPPER_1
#define NES_MAPPER_1

#include "..\NesBaseMapper.h"
#include <windows.h>

#define		BANKSIZE32	false
#define		BANKSIZE16	true

#define		BANKSIZE8	false
#define		BANKSIZE4	true

class NesMapper1 : public NesBaseMapper
{
public:	
	///////////////////////////////
	// Class Procedures
	///////////////////////////////
	NesMapper1(NesCartridge *linkCartridge, NesCPU *linkCPU, NesPPU *linkPPU);
	~NesMapper1() { };
	void initMemory();
	void write(unsigned __int16 addr, unsigned char data);

	// MMC1 specific methods
	void acceptData(unsigned char bytData, unsigned char bytPort);
	void transSwitchSpaces();
	void trans16to32();
	void trans32to16();


	///////////////////////////////
	// Class Data Members
	///////////////////////////////
	// Define a structure for holding cartridge status
	struct 
	{
			int	intBankSwitchOffset;	// D2 of reg 0: high or low switching? (0x8000 or 0xC000?)
			bool blnPRGBankSize;		// 16k or 32k bank switching (mapped to #defines)
			bool blnCHRBankSize;		// 4k or 8k bank switching (mapped to #defines)
	} status;
	
	// Define a structure for simulating a serial port interface
	struct
	{			
			unsigned char bytBitArray;
			unsigned char bytPtr;
	} SerialPort;
	

	int intBankMap16[2];				// Map of what 16k banks are loaded
	int intBankException;				// Used to track rare 32->16 bank switching occurence
	bool blnReg3Written;				// Boolean value for trapping peculiarty
	int intLastPort;
	byte regs[4];
};

#endif 
