//	UberNES Emulation Core
//  Copyright (C) 2004 Luke Molnar
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


//////////////////////////////////////////////////////
// opcodes.h -- Contains a listing of all 6502 opcodes
//////////////////////////////////////////////////////
// Opcode constants are named using "Opcode_Addressing Mode" convention
//////////////////////////////////////////////////////

// Assign constants to each addressing mode
#define     IMMEDIATE           0x0
#define     cABSOLUTE           0x1				// "c" added to offset annoying warnings
#define     ZERO_PAGE           0x2
#define     IMPLIED             0x3
#define     ACCUMULATOR         0x4
#define     ZERO_PAGE_X         0x5
#define     ZERO_PAGE_Y         0x6
#define     ABSOLUTE_X          0x7
#define     ABSOLUTE_Y          0x8
#define     INDIRECT_X          0x9
#define     INDIRECT_Y          0xA
#define     cRELATIVE           0xB				// "c" added to offset annoying warnings
#define     INDIRECT			0xC
#define     SPECIAL             0xD		// Don't want to do typical calcs with some modes/opcode (JSR Abs) 


// ADC
#define		ADC_IMMEDIATE		0x69
#define		ADC_ZERO_PAGE		0x65
#define		ADC_ZERO_PAGE_X		0x75
#define		ADC_ABSOLUTE		0x6D
#define		ADC_ABSOLUTE_X		0x7D
#define		ADC_ABSOLUTE_Y		0x79
#define		ADC_INDIRECT_X		0x61
#define		ADC_INDIRECT_Y		0x71

// AND
#define		AND_IMMEDIATE		0x29
#define		AND_ZERO_PAGE		0x25
#define		AND_ZERO_PAGE_X		0x35
#define		AND_ABSOLUTE		0x2D
#define		AND_ABSOLUTE_X		0x3D
#define		AND_ABSOLUTE_Y		0x39
#define		AND_INDIRECT_X		0x21
#define		AND_INDIRECT_Y		0x31

// ASL
#define		ASL_ACCUMULATOR		0x0A
#define		ASL_ZERO_PAGE		0x06
#define		ASL_ZERO_PAGE_X		0x16
#define		ASL_ABSOLUTE		0x0E
#define		ASL_ABSOLUTE_X		0x1E

// BCC
#define		BCC_RELATIVE		0x90

// BCS
#define		BCS_RELATIVE		0xB0

// BEW
#define		BEQ_RELATIVE		0xF0

// BIT
#define		BIT_ZERO_PAGE		0x24
#define		BIT_ABSOLUTE		0x2C

// BMI
#define		BMI_RELATIVE		0x30

// BNE
#define		BNE_RELATIVE		0xD0

// BPL
#define		BPL_RELATIVE		0x10

// BRK
#define		BRK_IMPLIED			0x00

// BVC
#define		BVC_RELATIVE		0x50

// BVS
#define		BVS_RELATIVE		0x70

// CLC
#define		CLC_IMPLIED			0x18

// CLD
#define		CLD_IMPLIED			0xD8

// CLI
#define		CLI_IMPLIED			0x58

// CLV
#define		CLV_IMPLIED			0xB8

// CMP
#define		CMP_IMMEDIATE		0xC9
#define		CMP_ZERO_PAGE		0xC5
#define		CMP_ZERO_PAGE_X		0xD5
#define		CMP_ABSOLUTE		0xCD
#define		CMP_ABSOLUTE_X		0xDD
#define		CMP_ABSOLUTE_Y		0xD9
#define		CMP_INDIRECT_X		0xC1
#define		CMP_INDIRECT_Y		0xD1

// CPX
#define		CPX_IMMEDIATE		0xE0
#define		CPX_ZERO_PAGE		0xE4
#define		CPX_ABSOLUTE		0xEC

// CPY
#define		CPY_IMMEDIATE		0xC0
#define		CPY_ZERO_PAGE		0xC4
#define		CPY_ABSOLUTE		0xCC

// DEC
#define		DEC_ZERO_PAGE		0xC6
#define		DEC_ZERO_PAGE_X		0xD6
#define		DEC_ABSOLUTE		0xCE
#define		DEC_ABSOLUTE_X		0xDE

// DEX
#define		DEX_IMPLIED			0xCA

// DEY
#define		DEY_IMPLIED			0x88

// EOR
#define		EOR_IMMEDIATE		0x49
#define		EOR_ZERO_PAGE		0x45
#define		EOR_ZERO_PAGE_X		0x55
#define		EOR_ABSOLUTE		0x4D
#define		EOR_ABSOLUTE_X		0x5D
#define		EOR_ABSOLUTE_Y		0x59
#define		EOR_INDIRECT_X		0x41
#define		EOR_INDIRECT_Y		0x51

// INC
#define		INC_ZERO_PAGE		0xE6
#define		INC_ZERO_PAGE_X		0xF6
#define		INC_ABSOLUTE		0xEE
#define		INC_ABSOLUTE_X		0xFE

// INX
#define		INX_IMPLIED			0xE8

// INY
#define		INY_IMPLIED			0xC8

// JMP
#define		JMP_ABSOLUTE		0x4C
#define		JMP_INDIRECT		0x6C

// JSR
#define		JSR_ABSOLUTE		0x20

// LDA
#define		LDA_IMMEDIATE		0xA9
#define		LDA_ZERO_PAGE		0xA5
#define		LDA_ZERO_PAGE_X		0xB5
#define		LDA_ABSOLUTE		0xAD
#define		LDA_ABSOLUTE_X		0xBD
#define		LDA_ABSOLUTE_Y		0xB9
#define		LDA_INDIRECT_X		0xA1
#define		LDA_INDIRECT_Y		0xB1


// LDX
#define		LDX_IMMEDIATE		0xA2
#define		LDX_ZERO_PAGE		0xA6
#define		LDX_ZERO_PAGE_Y		0xB6
#define		LDX_ABSOLUTE		0xAE
#define		LDX_ABSOLUTE_Y		0xBE

// LDY
#define		LDY_IMMEDIATE		0xA0
#define		LDY_ZERO_PAGE		0xA4
#define		LDY_ZERO_PAGE_X		0xB4
#define		LDY_ABSOLUTE		0xAC
#define		LDY_ABSOLUTE_X		0xBC

// LSR
#define		LSR_ACCUMULATOR		0x4A
#define		LSR_ZERO_PAGE		0x46
#define		LSR_ZERO_PAGE_X		0x56
#define		LSR_ABSOLUTE		0x4E
#define		LSR_ABSOLUTE_X		0x5E

// NOP
#define		NOP_IMPLIED			0xEA

// ORA
#define		ORA_IMMEDIATE		0x09
#define		ORA_ZERO_PAGE		0x05
#define		ORA_ZERO_PAGE_X		0x15
#define		ORA_ABSOLUTE		0x0D
#define		ORA_ABSOLUTE_X		0x1D
#define		ORA_ABSOLUTE_Y		0x19
#define		ORA_INDIRECT_X		0x01
#define		ORA_INDIRECT_Y		0x11

// PHA
#define		PHA_IMPLIED			0x48

// PHP
#define		PHP_IMPLIED			0x08

// PLA
#define		PLA_IMPLIED			0x68

// PLP
#define		PLP_IMPLIED			0x28

// ROL
#define		ROL_ACCUMULATOR		0x2A
#define		ROL_ZERO_PAGE		0x26
#define		ROL_ZERO_PAGE_X		0x36
#define		ROL_ABSOLUTE		0x2E
#define		ROL_ABSOLUTE_X		0x3E

// ROR
#define		ROR_ACCUMULATOR		0x6A
#define		ROR_ZERO_PAGE		0x66
#define		ROR_ZERO_PAGE_X		0x76
#define		ROR_ABSOLUTE		0x6E
#define		ROR_ABSOLUTE_X		0x7E

// RTI
#define		RTI_IMPLIED			0x40

// RTS
#define		RTS_IMPLIED			0x60

// SBC
#define		SBC_IMMEDIATE		0xE9
#define		SBC_ZERO_PAGE		0xE5
#define		SBC_ZERO_PAGE_X		0xF5
#define		SBC_ABSOLUTE		0xED
#define		SBC_ABSOLUTE_X		0xFD
#define		SBC_ABSOLUTE_Y		0xF9
#define		SBC_INDIRECT_X		0xE1
#define		SBC_INDIRECT_Y		0xF1

// SEC
#define		SEC_IMPLIED			0x38

// SED
#define		SED_IMPLIED			0xF8

// SEI
#define		SEI_IMPLIED			0x78

// STA
#define		STA_ZERO_PAGE		0x85
#define		STA_ZERO_PAGE_X		0x95
#define		STA_ABSOLUTE		0x8D
#define		STA_ABSOLUTE_X		0x9D
#define		STA_ABSOLUTE_Y		0x99
#define		STA_INDIRECT_X		0x81
#define		STA_INDIRECT_Y		0x91

// STX
#define		STX_ZERO_PAGE		0x86
#define		STX_ZERO_PAGE_Y		0x96
#define		STX_ABSOLUTE		0x8E

// STY
#define		STY_ZERO_PAGE		0x84
#define		STY_ZERO_PAGE_X		0x94
#define		STY_ABSOLUTE		0x8C

// TAX
#define		TAX_IMPLIED			0xAA

// TAY
#define		TAY_IMPLIED			0xA8

// TSX
#define		TSX_IMPLIED			0xBA

// TXA
#define		TXA_IMPLIED			0x8A

// TXS
#define		TXS_IMPLIED			0x9A

// TYA
#define		TYA_IMPLIED			0x98
